'use strict';

try {
    var baseCustomerHelpers = require('base/checkout/customer');
    // do stuff
} catch (ex) {}
var baseCheckout = require('base/checkout/checkout');
var billingHelpers = require('./billing');

var shippingHelpers = require('base/checkout/shipping');
var formHelpers = require('base/checkout/formErrors');
var scrollAnimate = require('base/components/scrollAnimate');

var gestpayHelper = require('./../gestpayHelper');

var gestpayFilteredAjaxCall = false;

var cc = {};

/**
 * Create the jQuery Checkout Plugin.
 *
 * This jQuery plugin will be registered on the dom element in checkout.isml with the
 * id of "checkout-main".
 *
 * The checkout plugin will handle the different state the user interface is in as the user
 * progresses through the varying forms such as shipping and payment.
 *
 * Billing info and payment info are used a bit synonymously in this code.
 *
 */
(function ($) {
    $.fn.checkout = function () { // eslint-disable-line
        var plugin = this;

        //
        // Collect form data from user input
        //
        var formData = {
            // Customer Data
            customer: {},

            // Shipping Address
            shipping: {},

            // Billing Address
            billing: {},

            // Payment
            payment: {},

            // Gift Codes
            giftCode: {}
        };
        //
        // The different states/stages of checkout
        //
        var checkoutStages = [
            'customer',
            'shipping',
            'payment',
            'placeOrder',
            'submitted'
        ];

        /**
         * Updates the URL to determine stage
         * @param {number} currentStage - The current stage the user is currently on in the checkout
         */
        function updateUrl(currentStage) {
            history.pushState(
                checkoutStages[currentStage],
                document.title,
                location.pathname +
                '?stage=' +
                checkoutStages[currentStage] +
                '#' +
                checkoutStages[currentStage]
            );
        }

        //
        // Local member methods of the Checkout plugin
        //
        var members = {

            // initialize the currentStage variable for the first time
            currentStage: 0,

            /**
             * Set or update the checkout stage (AKA the shipping, billing, payment, etc... steps)
             * @returns {Object} a promise
             */
            updateStage: function () {
                var stage = checkoutStages[members.currentStage];
                var defer = $.Deferred(); // eslint-disable-line

                if (stage === 'customer') {
                    //
                    // Clear Previous Errors
                    //
                    baseCustomerHelpers.methods.clearErrors();
                    //
                    // Submit the Customer Form
                    //
                    var customerFormSelector = baseCustomerHelpers.methods.isGuestFormActive() ? baseCustomerHelpers.vars.GUEST_FORM : baseCustomerHelpers.vars.REGISTERED_FORM;
                    var customerForm = $(customerFormSelector);
                    $.ajax({
                        url: customerForm.attr('action'),
                        type: 'post',
                        data: customerForm.serialize(),
                        success: function (data) {
                            if (data.redirectUrl) {
                                window.location.href = data.redirectUrl;
                            } else {
                                baseCustomerHelpers.methods.customerFormResponse(defer, data);
                            }
                        },
                        error: function (err) {
                            if (err.responseJSON && err.responseJSON.redirectUrl) {
                                window.location.href = err.responseJSON.redirectUrl;
                            }
                            // Server error submitting form
                            defer.reject(err.responseJSON);
                        }
                    });
                    return defer;
                } else if (stage === 'shipping') {
                    //
                    // Clear Previous Errors
                    //
                    formHelpers.clearPreviousErrors('.shipping-form');

                    //
                    // Submit the Shipping Address Form
                    //
                    var isMultiShip = $('#checkout-main').hasClass('multi-ship');
                    var formSelector = isMultiShip ?
                        '.multi-shipping .active form' : '.single-shipping .shipping-form';
                    var form = $(formSelector);

                    if (isMultiShip && form.length === 0) {
                        // disable the next:Payment button here
                        $('body').trigger('checkout:disableButton', '.next-step-button button');
                        // in case the multi ship form is already submitted
                        var url = $('#checkout-main').attr('data-checkout-get-url');
                        $.ajax({
                            url: url,
                            method: 'GET',
                            success: function (data) {
                                // enable the next:Payment button here
                                $('body').trigger('checkout:enableButton', '.next-step-button button');
                                if (!data.error) {
                                    $('body').trigger('checkout:updateCheckoutView', {
                                        order: data.order,
                                        customer: data.customer
                                    });
                                    defer.resolve();
                                } else if (data.message && $('.shipping-error .alert-danger').length < 1) {
                                    var errorMsg = data.message;
                                    var errorHtml = '<div class="alert alert-danger alert-dismissible valid-cart-error ' +
                                        'fade show" role="alert">' +
                                        '<button type="button" class="close" data-dismiss="alert" aria-label="Close">' +
                                        '<span aria-hidden="true">&times;</span>' +
                                        '</button>' + errorMsg + '</div>';
                                    $('.shipping-error').append(errorHtml);
                                    scrollAnimate($('.shipping-error'));
                                    defer.reject();
                                } else if (data.redirectUrl) {
                                    window.location.href = data.redirectUrl;
                                }
                            },
                            error: function () {
                                // enable the next:Payment button here
                                $('body').trigger('checkout:enableButton', '.next-step-button button');
                                // Server error submitting form
                                defer.reject();
                            }
                        });
                    } else {
                        var shippingFormData = form.serialize();

                        $('body').trigger('checkout:serializeShipping', {
                            form: form,
                            data: shippingFormData,
                            callback: function (data) {
                                shippingFormData = data;
                            }
                        });
                        // disable the next:Payment button here
                        $('body').trigger('checkout:disableButton', '.next-step-button button');
                        $.ajax({
                            url: form.attr('action'),
                            type: 'post',
                            data: shippingFormData,
                            success: function (data) {
                                // enable the next:Payment button here
                                $('body').trigger('checkout:enableButton', '.next-step-button button');
                                shippingHelpers.methods.shippingFormResponse(defer, data);
                            },
                            error: function (err) {
                                // enable the next:Payment button here
                                $('body').trigger('checkout:enableButton', '.next-step-button button');
                                if (err.responseJSON && err.responseJSON.redirectUrl) {
                                    window.location.href = err.responseJSON.redirectUrl;
                                }
                                // Server error submitting form
                                defer.reject(err.responseJSON);
                            }
                        });
                    }
                    return defer;
                } else if (stage === 'payment') {
                    //
                    // Submit the Billing Address Form
                    //

                    formHelpers.clearPreviousErrors('.payment-form');

                    var billingAddressForm = $('#dwfrm_billing .billing-address-block :input').serialize();

                    $('body').trigger('checkout:serializeBilling', {
                        form: $('#dwfrm_billing .billing-address-block'),
                        data: billingAddressForm,
                        callback: function (data) {
                            if (data) {
                                billingAddressForm = data;
                            }
                        }
                    });

                    var contactInfoForm = $('#dwfrm_billing .contact-info-block :input').serialize();

                    $('body').trigger('checkout:serializeBilling', {
                        form: $('#dwfrm_billing .contact-info-block'),
                        data: contactInfoForm,
                        callback: function (data) {
                            if (data) {
                                contactInfoForm = data;
                            }
                        }
                    });

                    var activeTabId = $('.tab-pane.active').attr('id');
                    var paymentInfoSelector = '#dwfrm_billing .' + activeTabId + ' .payment-form-fields :input';
                    var paymentInfoForm = $(paymentInfoSelector).serialize();

                    $('body').trigger('checkout:serializeBilling', {
                        form: $(paymentInfoSelector),
                        data: paymentInfoForm,
                        callback: function (data) {
                            if (data) {
                                paymentInfoForm = data;
                            }
                        }
                    });

                    var paymentForm = billingAddressForm + '&' + contactInfoForm + '&' + paymentInfoForm;

                    if ($('.data-checkout-stage').data('customer-type') === 'registered') {
                        // if payment method is credit card
                        if ($('.payment-information').data('payment-method-id') === 'CREDIT_CARD') {
                            if (!($('.payment-information').data('is-new-payment'))) {
                                var cvvCode = $('.saved-payment-instrument.' +
                                    'selected-payment .saved-payment-security-code').val();

                                if (cvvCode === '') {
                                    var cvvElement = $('.saved-payment-instrument.' +
                                        'selected-payment ' +
                                        '.form-control');
                                    cvvElement.addClass('is-invalid');
                                    scrollAnimate(cvvElement);
                                    defer.reject();
                                    return defer;
                                }

                                var $savedPaymentInstrument = $('.saved-payment-instrument' +
                                    '.selected-payment'
                                );

                                paymentForm += '&storedPaymentUUID=' +
                                    $savedPaymentInstrument.data('uuid');

                                paymentForm += '&securityCode=' + cvvCode;
                            }
                        }
                    }
                    // disable the next:Place Order button here
                    $('body').trigger('checkout:disableButton', '.next-step-button button');

                    $.ajax({
                        url: $('#dwfrm_billing').attr('action'),
                        method: 'POST',
                        data: paymentForm,
                        success: function (data) {
                            // enable the next:Place Order button here
                            $('body').trigger('checkout:enableButton', '.next-step-button button');
                            // look for field validation errors
                            if (data.error) {
                                if (data.fieldErrors.length) {
                                    data.fieldErrors.forEach(function (error) {
                                        if (Object.keys(error).length) {
                                            formHelpers.loadFormErrors('.payment-form', error);
                                        }
                                    });
                                }

                                if (data.serverErrors.length) {
                                    data.serverErrors.forEach(function (error) {
                                        $('.error-message').show();
                                        $('.error-message-text').text(error);
                                        scrollAnimate($('.error-message'));
                                    });
                                }

                                if (data.cartError) {
                                    window.location.href = data.redirectUrl;
                                }

                                defer.reject();
                            } else {
                                //
                                // Populate the Address Summary
                                //
                                $('body').trigger('checkout:updateCheckoutView', {
                                    order: data.order,
                                    customer: data.customer
                                });

                                if (data.renderedPaymentInstruments) {
                                    $('.stored-payments').empty().html(
                                        data.renderedPaymentInstruments
                                    );
                                }

                                if (data.customer.registeredUser &&
                                    data.customer.customerPaymentInstruments.length
                                ) {
                                    $('.cancel-new-payment').removeClass('checkout-hidden');
                                }

                                scrollAnimate();
                                defer.resolve(data);
                            }
                        },
                        error: function (err) {
                            // enable the next:Place Order button here
                            $('body').trigger('checkout:enableButton', '.next-step-button button');
                            if (err.responseJSON && err.responseJSON.redirectUrl) {
                                window.location.href = err.responseJSON.redirectUrl;
                            }
                        }
                    });

                    return defer;
                } else if (stage === 'placeOrder') {
                    // disable the placeOrder button here
                    $('body').trigger('checkout:disableButton', '.next-step-button button');
                    $.ajax({
                        url: $('.place-order').data('action'),
                        method: 'POST',
                        success: function (data) {
                            // enable the placeOrder button here
                            $('body').trigger('checkout:enableButton', '.next-step-button button');
                            if (data.error) {
                                if (data.cartError) {
                                    window.location.href = data.redirectUrl;
                                    defer.reject();
                                } else {
                                    // go to appropriate stage and display error message
                                    defer.reject(data);
                                }
                            } else {
                                var version = $("#versionSfra").attr("data-version");

                                if (version == 5) {
                                    var continueUrl = data.continueUrl;
                                    var urlParams = {
                                        ID: data.orderID,
                                        token: data.orderToken
                                    };

                                    continueUrl += (continueUrl.indexOf('?') !== -1 ? '&' : '?') +
                                        Object.keys(urlParams).map(function (key) {
                                            return key + '=' + encodeURIComponent(urlParams[key]);
                                        }).join('&');

                                    window.location.href = continueUrl;
                                    defer.resolve(data);
                                } else if (version == 6) {
                                    var redirect = $('<form>')
                                        .appendTo(document.body)
                                        .attr({
                                            method: 'POST',
                                            action: data.continueUrl
                                        });

                                    $('<input>')
                                        .appendTo(redirect)
                                        .attr({
                                            name: 'orderID',
                                            value: data.orderID
                                        });

                                    $('<input>')
                                        .appendTo(redirect)
                                        .attr({
                                            name: 'orderToken',
                                            value: data.orderToken
                                        });

                                    redirect.submit();
                                    defer.resolve(data);
                                }
                            }
                        },
                        error: function () {
                            // enable the placeOrder button here
                            $('body').trigger('checkout:enableButton', $('.next-step-button button'));
                        }
                    });

                    return defer;
                }
                var p = $('<div>').promise(); // eslint-disable-line
                setTimeout(function () {
                    p.done(); // eslint-disable-line
                }, 500);
                return p; // eslint-disable-line
            },

            /**
             * Initialize the checkout stage.
             */
            initialize: function () {
                // set the initial state of checkout
                members.currentStage = checkoutStages
                    .indexOf($('.data-checkout-stage').data('checkout-stage'));
                $(plugin).attr('data-checkout-stage', checkoutStages[members.currentStage]);

                $('body').on('click', '.submit-customer-login', function (e) {
                    e.preventDefault();
                    members.nextStage();
                });

                $('body').on('click', '.submit-customer', function (e) {
                    e.preventDefault();
                    members.nextStage();
                });

                //
                // Handle Payment option selection
                //
                $('input[name$="paymentMethod"]', plugin).on('change', function () {
                    $('.credit-card-form').toggle($(this).val() === 'CREDIT_CARD');
                    $('.credit-card-form.checkout-hidden').removeAttr("style");
                });

                //
                // Handle Next State button click
                //
                $(plugin).on('click', '.next-step-button button', function () {
                    members.nextStage();
                });

                //
                // Handle Edit buttons on shipping and payment summary cards
                //
                $('.customer-summary .edit-button', plugin).on('click', function () {
                    members.gotoStage('customer');
                });

                $('.shipping-summary .edit-button', plugin).on('click', function () {
                    if (!$('#checkout-main').hasClass('multi-ship')) {
                        $('body').trigger('shipping:selectSingleShipping');
                    }

                    members.gotoStage('shipping');
                });

                $('.payment-summary .edit-button', plugin).on('click', function () {
                    members.gotoStage('payment');
                });

                //
                // remember stage (e.g. shipping)
                //
                updateUrl(members.currentStage);

                //
                // Listen for foward/back button press and move to correct checkout-stage
                //
                $(window).on('popstate', function (e) {
                    //
                    // Back button when event state less than current state in ordered
                    // checkoutStages array.
                    //
                    if (e.state === null ||
                        checkoutStages.indexOf(e.state) < members.currentStage) {
                        members.handlePrevStage(false);
                    } else if (checkoutStages.indexOf(e.state) > members.currentStage) {
                        // Forward button  pressed
                        members.handleNextStage(false);
                    }
                });

                //
                // Set the form data
                //
                plugin.data('formData', formData);
            },

            /**
             * The next checkout state step updates the css for showing correct buttons etc...
             */
            nextStage: function () {
                var promise = members.updateStage();

                promise.done(function () {
                    // Update UI with new stage
                    $('.error-message').hide();
                    members.handleNextStage(true);
                });

                promise.fail(function (data) {
                    // show errors
                    if (data) {
                        if (data.errorStage) {
                            members.gotoStage(data.errorStage.stage);

                            if (data.errorStage.step === 'billingAddress') {
                                var $billingAddressSameAsShipping = $(
                                    'input[name$="_shippingAddressUseAsBillingAddress"]'
                                );
                                if ($billingAddressSameAsShipping.is(':checked')) {
                                    $billingAddressSameAsShipping.prop('checked', false);
                                }
                            }
                        }

                        if (data.errorMessage) {
                            $('.error-message').show();
                            $('.error-message-text').text(data.errorMessage);
                        }
                    }
                });
            },

            /**
             * The next checkout state step updates the css for showing correct buttons etc...
             *
             * @param {boolean} bPushState - boolean when true pushes state using the history api.
             */
            handleNextStage: function (bPushState) {
                if (members.currentStage < checkoutStages.length - 1) {
                    // move stage forward
                    members.currentStage++;

                    //
                    // show new stage in url (e.g.payment)
                    //
                    if (bPushState) {
                        updateUrl(members.currentStage);
                    }
                }

                // Set the next stage on the DOM
                $(plugin).attr('data-checkout-stage', checkoutStages[members.currentStage]);
            },

            /**
             * Previous State
             */
            handlePrevStage: function () {
                if (members.currentStage > 0) {
                    // move state back
                    members.currentStage--;
                    updateUrl(members.currentStage);
                }

                $(plugin).attr('data-checkout-stage', checkoutStages[members.currentStage]);
            },

            /**
             * Use window history to go to a checkout stage
             * @param {string} stageName - the checkout state to goto
             */
            gotoStage: function (stageName) {
                members.currentStage = checkoutStages.indexOf(stageName);
                updateUrl(members.currentStage);
                $(plugin).attr('data-checkout-stage', checkoutStages[members.currentStage]);
            }
        };

        //
        // Initialize the checkout
        //
        members.initialize();

        return this;
    };
}(jQuery));


[billingHelpers].forEach(function (library) {
    Object.keys(library).forEach(function (item) {
        if (typeof library[item] === 'object') {
            baseCheckout[item] = $.extend({}, baseCheckout[item], library[item]);
        } else {
            baseCheckout[item] = library[item];
        }
    });
});

$(document).ready(function () {
    $('.payment-options .nav-item').on('click', function (e) {
        var methodID = $(this).data('method-id');
        $('.payment-information').data('paymentMethodId', methodID);
        $(`input[name$="paymentMethod"][checked]`).removeAttr('checked');
        $(`input[name$="paymentMethod"]:checked`).removeProp('checked');
        $(`input[value="${methodID}"]`).attr('checked', true);
        $(`input[value="${methodID}"]`).prop('checked', true);
        $(`input[value="${methodID}"]`).trigger('change');
    }).find('.nav-link.active').parent().trigger('click');

    var gestpay3dSecureCallback = $('#gestpay3dSecureCallback');

    if (gestpay3dSecureCallback.length > 0) {
        var gestpayData = gestpay3dSecureCallback.data();

        GestPay.CreatePaymentPage(
            gestpayData.shoplogin,
            gestpayData.cryptdecryptstring,
            function (createPaymentPageResult) {
                createPaymentPageResult.cryptDecryptString =
                    gestpayData.cryptdecryptstring;
                createPaymentPageResult.orderId = gestpayData.orderid;
                createPaymentPageResult.orderToken = gestpayData.ordertoken;
                $.ajax({
                    url: gestpayData.check3dsecurepageurl,
                    method: 'POST',
                    data: createPaymentPageResult
                }).done(function (data) {
                    if (data.error) {
                        window.location = gestpayData.errorurl;
                    } else {
                        var sendPayment3dRequest = {
                            TransKey: gestpayData.transkey.toString(),
                            PARes: gestpayData.pares
                        };
                        GestPay.SendPayment(sendPayment3dRequest, function (
                            sendPaymentResult
                        ) {
                            sendPaymentResult.cryptDecryptString =
                                gestpayData.cryptdecryptstring;
                            sendPaymentResult.orderId = gestpayData.orderid;
                            sendPaymentResult.orderToken = gestpayData.ordertoken;
                            $.ajax({
                                url: gestpayData.palceorderurl,
                                method: 'POST',
                                data: sendPaymentResult
                            }).done(function (sendPaymentResponse) {

                                if (sendPaymentResponse.version == 6) {
                                    $('<form class="d-none" action="' + sendPaymentResponse.redirectUrl + '" method="POST">' +
                                        '<input type="hidden" name="orderID" value="' + sendPaymentResult.orderId + '" />' +
                                        '<input type="hidden" name="orderToken" value="' + sendPaymentResult.orderToken + '" />' +
                                        '</form>').appendTo($(document.body)).submit();

                                } else {
                                    window.location = sendPaymentResponse.redirectUrl;
                                }
                            });
                        });
                    }
                });
            }
        );
    } else {
        var plugin = $('#checkout-main');
        var isGestPayIframeMode = $(plugin).data('gestpayIframemode');

        if (isGestPayIframeMode) {
            var gestpayEncryptUrl = $(plugin).data('gestpayUrl');

            var members = {
                checkGestpayResult: function (fields, Result, formContainer) {
                    if (Result.ErrorCode != 0) {
                        gestpayFilteredAjaxCall = false;
                        var error = {};
                        $.each(fields, function (fieldIndex) {
                            error[fields[fieldIndex]] =
                                $(plugin).data('errormsg-' + Result.ErrorCode) ||
                                $(plugin).data('errormsgDefault') ||
                                Result.ErrorDescription;
                        });
                        formHelpers.loadFormErrors(
                            $('.payment-information').data('is-new-payment') ?
                            '.payment-form' :
                            '.stored-payments',
                            error
                        );
                    }
                },
                getCreditCardNumber: function () {
                    return $('input[name$=cardNumber]');
                },
                getSecurityCode: function () {
                    return $('.payment-information').data('is-new-payment') ?
                        $('input[name$="_securityCode"]') :
                        $('input[name$="saved-payment-security-code"]');
                },
                getExpirationMonth: function () {
                    return $('select[name$="_expirationMonth"]');
                },
                getExpirationYear: function () {
                    return $('select[name$="_expirationYear"]');
                },
                getCreditCardFieldsEmail: function () {
                    return $('input[name$="_creditCardFields_email"]');
                },
                isNewPayment: function () {
                    return $('.payment-information').data('is-new-payment');
                }
            };

            // force customer to input again the credit card number and the cvv when the checkout is refreshed
            var stage = plugin.data('checkoutStage');
            if (['payment', 'placeOrder'].indexOf(stage) > -1 &&
                $('input[name$="paymentMethod"]:checked,input[name$="paymentMethod"][checked]').val() === 'CREDIT_CARD' &&
                !/\d{12,16}/.test(members.getCreditCardNumber().data('cleave').getRawValue())) {
                members.getCreditCardNumber().val('');
                members.getSecurityCode().val('');
                $('.payment-summary .edit-button', plugin).click();
                members.getCreditCardNumber().focus();
            }

            $.ajaxPrefilter(function (options, originalOptions, jqXHR) {
                if (gestpayFilteredAjaxCall) {
                    gestpayFilteredAjaxCall = false;
                    return;
                }

                if (options.url === $('#dwfrm_billing').attr('action') && $('input[name$="paymentMethod"]:checked,input[name$="paymentMethod"][checked]').val() === 'CREDIT_CARD') {
                    gestpayFilteredAjaxCall = true;
                    options.data = originalOptions.data.replace(/(cardNumber=)([0-9 %]*)(\d)(\d)(\d)(\d)/gi, '$1XXXXXXXXXXXX$3$4$5$6').replace(/(securityCode=)([0-9]*)/gi, '$1XXX');

                    var queryString = '';
                    if (gestpayHelper.isSavedCard()) {
                        queryString += '?savedcard=true';
                    } else if (gestpayHelper.isSavingCard()) {
                        queryString += '?savingcard=true';
                    }

                    options.url = gestpayEncryptUrl + queryString;
                    options.type = 'GET';

                    options.success = function (data) {
                        if (data.transactionResult === 'OK') {
                            if (!gestpayHelper.isSavedCard()) {
                                GestPay.CreatePaymentPage(
                                    data.shopLogin,
                                    data.cryptDecryptString,
                                    function (CreatePaymentPageResult) {
                                        if (CreatePaymentPageResult.ErrorCode == 10) {
                                            // check CVV
                                            GestPay.checkCVV(
                                                members.getSecurityCode().val(),
                                                function (Result) {
                                                    cc.ccv = members.getSecurityCode().val();
                                                    members.checkGestpayResult(
                                                        [members.getSecurityCode().attr('name')],
                                                        Result
                                                    );
                                                    // check CC date and email for new payment
                                                    if (members.isNewPayment()) {
                                                        // check CC
                                                        GestPay.checkCC(
                                                            members
                                                            .getCreditCardNumber()
                                                            .data('cleave')
                                                            .getRawValue(),
                                                            function (ResultCC) {
                                                                cc.cc = members
                                                                    .getCreditCardNumber()
                                                                    .data('cleave')
                                                                    .getRawValue();
                                                                members.checkGestpayResult(
                                                                    [members.getCreditCardNumber().attr('name')],
                                                                    ResultCC
                                                                );
                                                                // check exp date
                                                                GestPay.checkEXPDATE(
                                                                    (
                                                                        '00' + members.getExpirationMonth().val()
                                                                    ).slice(-2),
                                                                    members
                                                                    .getExpirationYear()
                                                                    .val()
                                                                    .slice(-2),
                                                                    function (ResultEXPDATE) {
                                                                        members.checkGestpayResult(
                                                                            [
                                                                                members
                                                                                .getExpirationMonth()
                                                                                .attr('name'),
                                                                                members.getExpirationYear().attr('name')
                                                                            ],
                                                                            ResultEXPDATE
                                                                        );
                                                                        // check email
                                                                        GestPay.checkEMAIL(
                                                                            members.getCreditCardFieldsEmail().val(),
                                                                            function (ResultEMAIL) {
                                                                                members.checkGestpayResult(
                                                                                    [
                                                                                        members
                                                                                        .getCreditCardFieldsEmail()
                                                                                        .attr('name')
                                                                                    ],
                                                                                    ResultEMAIL
                                                                                );
                                                                                if (
                                                                                    $(
                                                                                        '.payment-form *.is-invalid, .stored-payments *.is-invalid'
                                                                                    ).length === 0
                                                                                ) {
                                                                                    $.ajax(originalOptions);
                                                                                }
                                                                            }
                                                                        );
                                                                    }
                                                                );
                                                            }
                                                        );
                                                    } else if (
                                                        $(
                                                            '.payment-form *.is-invalid, .stored-payments *.is-invalid'
                                                        ).length === 0
                                                    ) {
                                                        $.ajax(originalOptions);
                                                    }
                                                }
                                            );
                                            $('body').trigger('checkout:enableButton', '.next-step-button button');
                                        } else {
                                            $('.error-message').show();
                                            $('.error-message-text').text(
                                                $(plugin).data('errormsg-default')
                                            );
                                            originalOptions.error && originalOptions.error();
                                        }
                                    }
                                );
                            } else if (
                                $('.payment-form *.is-invalid, .stored-payments *.is-invalid')
                                .length === 0
                            ) {
                                $.ajax(originalOptions);
                            }
                        } else {
                            $('.error-message').show();
                            $('.error-message-text').text($(plugin).data('errormsg-default'));
                            originalOptions.error && originalOptions.error();
                        }
                    };
                }

                if (
                    options.url === $('.place-order').data('action') &&
                    $('input[name$="paymentMethod"]:checked,input[name$="paymentMethod"][checked]').val() === 'CREDIT_CARD' &&
                    members.isNewPayment()
                ) {
                    gestpayFilteredAjaxCall = true;
                    options.success = function (data) {
                        var isMultiShip = $('#checkout-main').hasClass('multi-ship');
                        var formSelector = isMultiShip ?
                            '.multi-shipping .active form' :
                            '.single-shipping form';
                        var form = $(formSelector);
                        if (!data.error) {
                            var orderId = data.orderID;
                            var orderToken = data.orderToken;
                            var cryptDecryptString = data.cryptDecryptString;
                            GestPay.CreatePaymentPage(
                                data.shopLogin,
                                cryptDecryptString,
                                function (Result) {
                                    if (Result.ErrorCode == 10) {
                                        GestPay.SendPayment({
                                                CC: cc.cc,
                                                EXPMM: (
                                                    '00' + members.getExpirationMonth().val()
                                                ).slice(-2),
                                                EXPYY: members
                                                    .getExpirationYear()
                                                    .val()
                                                    .slice(-2),
                                                CVV2: cc.ccv,
                                                Name: $('input[name$=_firstName]', form).val() +
                                                    ' ' +
                                                    $('input[name$=_lastName]', form).val(),
                                                Email: members.getCreditCardFieldsEmail().val()
                                            },
                                            function (ResultOfSendPayment) {
                                                ResultOfSendPayment.cryptDecryptString = cryptDecryptString;
                                                ResultOfSendPayment.orderId = orderId;
                                                ResultOfSendPayment.orderToken = orderToken;

                                                $.ajax({
                                                    url: $('.place-order-iframe').data('action'),
                                                    data: ResultOfSendPayment,
                                                    method: 'POST'
                                                }).done(function (resultOfPlaceOrderIFrame) {
                                                    if (resultOfPlaceOrderIFrame.redirectUrl) {
                                                        // force redirect
                                                        resultOfPlaceOrderIFrame.error = true;
                                                        resultOfPlaceOrderIFrame.cartError = true;
                                                    }
                                                    originalOptions.success(resultOfPlaceOrderIFrame);
                                                });
                                            }
                                        );
                                    } else {
                                        $('.error-message').show();
                                        $('.error-message-text').text(
                                            $(plugin).data('errormsg-default')
                                        );
                                        return originalOptions.success({
                                            error: true
                                        });
                                    }
                                }
                            );
                        } else {
                            return originalOptions.success(data);
                        }
                    };
                }
            });
        }
    }
});

module.exports = baseCheckout;